////////////////////////////////////////////////////////////////////////////
//   Copyright 2013  Teradici Corporation
//
//   This program is free software; you can redistribute it and/or modify it
//   under the terms and conditions of the GNU General Public License,
//   version 2, as published by the Free Software Foundation.
//
////////////////////////////////////////////////////////////////////////////
/// \file
/// \brief
///     Memory Module Internal
///
///     This file defines the internal memory module API prototypes and
///     definitions.
///
////////////////////////////////////////////////////////////////////////////

#pragma once

// ---- Include Files -------------------------------------------------------

#include "tera_mem.h"

// ---- Constants and Types -------------------------------------------------

// Defines for memory pool size
#define TERA_CHUNK_SIZE    PAGE_SIZE
#define TERA_NUM_CHUNKS    256
#define TERA_MEM_POOL_SIZE (TERA_NUM_CHUNKS * TERA_CHUNK_SIZE)

// --------------------------------------------------------------------------
//  sTERA_MEM_POOL
///     Memory pool
//
//  Description:
///     This structure defines a memory pool. This structure contains a pointer
///     to a large contiguous region of memory which forms the pool. The region
///     is partitioned into individual chunks.
///
typedef struct sTERA_MEM_POOL_STRUCT
{
    sTERA_MEM_DMA *buf;     ///< Handle to DMA'able memory
    UINT32         size;    ///< Total size of memory pool
    sTERA_MEM_BUF  chunk[TERA_NUM_CHUNKS];
} sTERA_MEM_POOL;

// --------------------------------------------------------------------------
//  sTERA_MEM_CBLK
///     Memory module control block
//
//  Description:
///     This structure defines the memory pool module control block. The
///     control block contains the state flag, DMA'able memory allocation
///     handle, and a memory pool.
///
typedef struct sTERA_MEM_CBLK_STRUCT
{
    UINT8          initialized;     ///< Initialized flag

//    WDFDMAENABLER  dma_enabler;     ///< DMA enabler for allocating common buffers
    sTERA_MEM_POOL pool;            ///< Memory pool
    UINT32         mem_alloc_cnt;   ///< Memory allocation counter
    UINT32         mem_free_cnt;    ///< Memory free counter
    UINT32         buf_alloc_cnt;   ///< Buffer allocation counter
    UINT32         buf_free_cnt;    ///< Buffer free counter
} sTERA_MEM_CBLK;

// Declare an accessor function to get the context area
//WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(sTERA_MEM_CBLK, tera_mem_cblk_get);

// ---- Variable Externs ----------------------------------------------------

// ---- Function Prototypes -------------------------------------------------

